#include "game.h"
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

#define RH_WIDTH 6
#define RH_HEIGHT 6

struct game_s{
  int width;
  int height;
  int nbMoves;
  int nbPieces;
  piece *pieces;
};

game new_game_hr (int nb_pieces, piece *pieces){
  return new_game(RH_WIDTH, RH_HEIGHT, nb_pieces, pieces);
}


void delete_game (game g){
  // Appelle d'abord le delete de toutes les pieces avant de
  // delete la liste puis lui même
  for(int i = 0; i < game_nb_pieces(g); i++){
    delete_piece(g->pieces[i]);
  }
  free(g->pieces);
  free(g);
}

void copy_game(cgame src, game dst){
  // A chaque fois que ce la est necessaire, on verifie que les games existent
  assert(dst != NULL && src != NULL);
  
  dst->nbMoves = game_nb_moves(src);
  dst->nbPieces = game_nb_pieces(src);
  dst->width = game_width(src);
  dst->height = game_height(src);
  dst->pieces = (piece*) malloc(sizeof(piece) * game_nb_pieces(src));

  for(int i = 0; i < game_nb_pieces(src); i++){
    dst->pieces[i] = (piece) malloc(sizeof(piece));
    assert(dst->pieces[i] != NULL);
    copy_piece(game_piece(src, i), dst->pieces[i]);
  }
}

int game_nb_pieces(cgame g){
  assert(g != NULL);
  return g->nbPieces;
}

cpiece game_piece(cgame g, int piece_num){
  // On check que la valeur de piece_num ne soit pas absurde
  assert(g != NULL);
  assert((piece_num >= 0) && (piece_num < game_nb_pieces(g)));
  return g->pieces[piece_num];
}

bool game_over_hr(cgame g){
  assert(g != NULL);
  return get_x(game_piece(g, 0)) == 4 && get_y(game_piece(g, 0)) == 3;
}

bool play_move(game g, int piece_num, dir d, int distance){
  // La fonction crée en premier lieu une piece de test
  // pour verifier la validité du déplacement
  piece piece_test = new_piece_rh(0, 0, true, true);
  copy_piece(game_piece(g, piece_num), piece_test);

  // Vérifications de la conditions 2 d'un move valide (+ valeurs valides)
  bool test_move = (distance > 0) && (piece_num >= 0) && (piece_num < game_nb_pieces(g)-1);
  test_move = (can_move_x(game_piece(g, piece_num)) && (d == LEFT || d == RIGHT)) ||
    (can_move_y(game_piece(g, piece_num)) && (d == UP || d == DOWN));

  // On effectue le déplacement voulu sur la piece test case par case
  // On s'assure ainsi de ne pas faire de saute-mouton
  int i = 0;
  while(test_move && i < distance){
    move_piece(piece_test, d, 1);
    if(d == UP || d == DOWN){
      test_move = get_y(game_piece(g, piece_num)) != get_y(piece_test);
    }
    else{
      test_move = get_x(game_piece(g, piece_num)) != get_x(piece_test);
    }
    // Vérification condition 1 d'un move valide
    if(test_move){
      test_move = (get_x(piece_test) >= 0) && (get_y(piece_test) >= 0)
	&& ((get_x(piece_test) + get_width(piece_test) - 1) < game_width(g))
	&& ((get_y(piece_test) + get_height(piece_test) -1) < game_height(g));
    }
    
    int j = 0;
    // Vérification de la condition 3
    while(test_move && j < game_nb_pieces(g)){
      if(j != piece_num){
	test_move = !intersect(game_piece(g, j), piece_test);
      }
      j++;
    }
    i++;
  }
  // Si le move est valide, on déplace la véritable piece.
  if(test_move){
    move_piece(g->pieces[piece_num], d, distance);
    g->nbMoves += distance;
  }
  delete_piece(piece_test);
  return test_move;
}

int game_nb_moves(cgame g){
  assert(g != NULL);
  return g->nbMoves;
}

////////////////// version 2 //////////////////////

game new_game(int width, int height, int nb_pieces, piece *pieces){
  game g = (game)malloc(sizeof(struct game_s));
  assert(g!=NULL);
  
  g->pieces = (piece*) malloc(sizeof(piece) * nb_pieces);
  g->width = width;
  g->height = height;
  g->nbMoves = 0;
  g->nbPieces = nb_pieces;
  assert(g->pieces != NULL);

  // Ici on s'assure qu'aucune piece ne sorte du terrain au cas ou
  // la taille diffère des niveaux codés en dur actuellement.
  // Condition a déplacer sur la création de la piece elle même
  // lorsque les niveaux seront générés aléatoirement.
  for(int i = 0; i < nb_pieces; i++){
    if(get_x(pieces[i]) < (game_width(g) - get_width(pieces[i]) + 1) &&
       (get_y(pieces[i]) + get_height(pieces[i]) - 1) < game_height(g)){
	g->pieces[i] = new_piece_rh(0,0,true,true);
	copy_piece(pieces[i], g->pieces[i]);
    }else{
      fprintf(stderr, "La taille du terrain est trop petite \n");
      exit(EXIT_FAILURE);
      }
  }
  
  return g;

}

int game_width(cgame g){
  return g->width;
}

int game_height(cgame g){
  return g->height;
}

int game_square_piece(game g, int x, int y){
  if(game_nb_pieces(g) < 1){
    return -1;
  }
  piece test_piece = new_piece(x, y, 1, 1, false, false);
  for(int i = 0; i < game_nb_pieces(g); i++){
    if(intersect(game_piece(g, i), test_piece)){
      delete_piece(test_piece);
      return i;
    }
  }
  delete_piece(test_piece);
  return -1;
}
