#include <stdlib.h>
#include <stdio.h>
#include <time.h>


#include "game.h"
#include "piece.h"

#define NB_PIECES 4

game g;
piece *pieces;

void set_up(){
  pieces = (piece*)malloc(sizeof(piece) * NB_PIECES);
  pieces[0] = new_piece_rh(3, 3, true, true);
  pieces[1] = new_piece_rh(3, 0, true, false);
  pieces[2] = new_piece_rh(4, 1, true, true);
  pieces[3] = new_piece_rh(5, 3, false, false);
  g = new_game_hr(NB_PIECES, pieces);
}

bool test_new_game(){
  bool result = true;
  game g_test = new_game_hr(NB_PIECES, pieces);
  if (g_test == NULL){
    result = false;
  }
  delete_game(g_test);
  printf("test_new_game : %d \n", result);
  return result;
}

bool test_nb_pieces(){
  bool result = true;
  if (game_nb_pieces((cgame)g)!=NB_PIECES){
    result = false;
  }
  printf("test_nb_pieces : %d \n", result);
  return result;
}

bool test_game_piece(){
  bool result = true;
  for (int i=0; i<NB_PIECES; i++){
    result = result && get_x(game_piece(g, i)) == get_x(pieces[i]);
    result = result && get_y(game_piece(g, i)) == get_y(pieces[i]);
    result = result && get_height(game_piece(g,i)) == get_height(pieces[i]);
    result = result && get_width(game_piece(g,i)) == get_width(pieces[i]);
    result = result && is_horizontal(game_piece(g, i)) == is_horizontal(pieces[i]);
  }
  printf("test_game_piece : %d \n", result);
  return result;
}

bool test_game_nb_moves(){
  bool result = true;
  if (game_nb_moves((cgame)g)!=0){
    result = false;
  }
  printf("test_nb_moves : %d \n", result);
  return result;
}

bool test_copy_game(){
  bool result = true;
  piece *p_copy = (piece*)malloc(sizeof(piece));
  p_copy[0] = new_piece_rh(5,1,true,true);
  game g_copy = new_game_hr(0, p_copy);
  copy_game((cgame)g, g_copy);
  result = (NB_PIECES == game_nb_pieces(g_copy));
  result = (game_nb_moves((cgame)g_copy) == game_nb_moves((cgame)g));
  for (int i=0; i<NB_PIECES; i++){
    result = result && get_x(game_piece(g_copy, i)) == get_x(game_piece(g, i));
    result = result && get_y(game_piece(g_copy, i)) == get_y(game_piece(g , i));
    result = result && get_height(game_piece(g_copy, i)) == get_height(game_piece(g, i));
    result = result && get_width(game_piece(g_copy, i)) == get_width(game_piece(g, i));
    result = result && is_horizontal(game_piece(g_copy, i)) == is_horizontal(game_piece(g, i));
  }
  delete_game(g_copy);
  printf("test_copy_game : %d", result);
  return result;
}

bool test_game_over(){
  bool result1;
  bool result2;
  result1 = !(game_over_hr((cgame)g));
  piece *p_test = (piece*)malloc(sizeof(piece));
  p_test[0] = new_piece_rh(4,3,true,true);
  game g_test = new_game_hr(1,p_test);
  result2 = game_over_hr((cgame)g_test);
  delete_game(g_test);
  printf("test_game_over : %d", result1 && result2);
  return (result1 && result2);
}

bool test_play_move(){
  bool result = true;
  for (int i=0; i<NB_PIECES; i++){
    if (is_horizontal(pieces[i])){
      if (!play_move(g, i, LEFT, 1)){ //si la piece ne peut pas bouger, c'est qu'il y'a une autre piece ou bord du jeu atteint ; sinon, il y a un bug
	for (int j=0; j<NB_PIECES; j++){
	  if(i!=j){
	    result = intersect(pieces[i], pieces[j]); //si il y a intersect, tout est ok
	  }
	}
      }
      if (!play_move(g, i, RIGHT, 1)){
	for (int j=0; j<NB_PIECES; j++){
	  if(i!=j){
	    result = intersect(pieces[i], pieces[j]); //si il y a intersect, tout est ok
	  }
	}
      }
      result = !play_move(g, i, UP, 1);
      result = !play_move(g, i, DOWN, 1);
    }
    else{
      if (!play_move(g, i, UP, 1)){
	for (int j=0; j<NB_PIECES; j++){
	  if(i!=j){
	    result = intersect(pieces[i], pieces[j]);
	  }
	}
      }
      if (!play_move(g, i, DOWN, 1)){
	for (int j=0; j<NB_PIECES; j++){
	  if (i!=j){
	    result = intersect(pieces[i], pieces[j]);
	  }
	}
      }
      result = !play_move(g, i, LEFT, 1);
      result = !play_move(g, i, RIGHT, 1);
    }
  }
  printf("test_play_move : %d", result);
  return result;
}
    
int main(int argc, char * argv[]){
  bool result = true;
  set_up();
  result = result && test_new_game();
  result = result && test_nb_pieces();
  result = result && test_game_piece();
  result = result && test_game_nb_moves();
  result = result && test_copy_game();
  result = result && test_game_over();
  result = result && test_play_move();
  delete_game(g);
  if (result){
    printf("Tests OK \n");
    return EXIT_SUCCESS;
  }
  else{
    printf("Tests failed \n");
    return EXIT_FAILURE;
  }
}  
  
