#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include "game_ar.h"
#include "piece.h"

#define NB_PIECES 4
#define AR_WIDTH 10
#define AR_HEIGHT 10

game g;
piece *pieces;

void set_up(){
  pieces = (piece*)malloc(sizeof(piece) * NB_PIECES);
  pieces[0] = new_piece(3, 3, 1, 1, true, true);
  pieces[1] = new_piece(3, 0, 1, 1, true, false);
  pieces[2] = new_piece(4, 1, 1, 1, true, true);
  pieces[3] = new_piece(5, 3, 1, 1, false, false);
  g = new_game(AR_WIDTH, AR_HEIGHT, NB_PIECES, pieces);
}

bool test_new_game(){
  bool result = true;
  game g_test = new_game(AR_WIDTH, AR_HEIGHT, NB_PIECES, pieces);
  if (g_test == NULL){
    result = false;
  }
  if (game_width((cgame)g_test)!=AR_WIDTH){
    printf("Erreur sur l'abscisse du terrain");
    result = false;
  }
  if (game_height((cgame)g_test)!=AR_HEIGHT){
    printf("Erreur sur l'ordonnee du terrain");
    result = false;
  }
  if (game_nb_moves((cgame)g_test)!=0){
    printf("Erreur sur le nombre de deplacements");
    result = false;
  }
  if (game_nb_pieces((cgame)g_test)!=NB_PIECES){
    printf("Erreur sur le nombre de pièces");
    result = false;
  }
  delete_game(g_test);
  printf("test_new_game : %d \n", result);
  return result;
}

bool test_nb_pieces(){
  bool result = true;
  if (game_nb_pieces((cgame)g)!=NB_PIECES){
    result = false;
  }
  printf("test_nb_pieces : %d \n", result);
  return result;
}

bool test_game_piece(){
  bool result = true;
  for (int i=0; i<NB_PIECES; i++){
    result = result && get_x(game_piece(g, i)) == get_x(pieces[i]);
    result = result && get_y(game_piece(g, i)) == get_y(pieces[i]);
    result = result && get_height(game_piece(g,i)) == get_height(pieces[i]);
    result = result && get_width(game_piece(g,i)) == get_width(pieces[i]);
    result = result && is_horizontal(game_piece(g, i)) == is_horizontal(pieces[i]);
  }
  printf("test_game_piece : %d \n", result);
  return result;
}

bool test_game_nb_moves(){
  bool result = true;
  if (game_nb_moves((cgame)g)!=0){
    result = false;
  }
  printf("test_nb_moves : %d \n", result);
  return result;
}

bool test_copy_game(){
  bool result = true;
  piece p_copy[1];
  game g_copy = new_game(AR_WIDTH, AR_HEIGHT, 0, p_copy);
  copy_game((cgame)g, g_copy);
  result = (NB_PIECES == game_nb_pieces(g_copy));
  result = (game_nb_moves((cgame)g_copy) == game_nb_moves((cgame)g));
  result = (game_width((cgame)g) == AR_WIDTH);
  result = (game_height((cgame)g) == AR_HEIGHT);
  for (int i=0; i<NB_PIECES; i++){
    result = result && get_x(game_piece(g_copy, i)) == get_x(game_piece(g, i));
    result = result && get_y(game_piece(g_copy, i)) == get_y(game_piece(g , i));
    result = result && get_height(game_piece(g_copy, i)) == get_height(game_piece(g, i));
    result = result && get_width(game_piece(g_copy, i)) == get_width(game_piece(g, i));
    result = result && is_horizontal(game_piece(g_copy, i)) == is_horizontal(game_piece(g, i));
  }
  delete_game(g_copy);
  printf("test_copy_game : %d\n", result);
  return result;
}

bool test_game_over_ar(){
  bool result1;
  bool result2;
  result1 = !(game_over_ar((cgame)g));
  piece p_test[1];
  p_test[0] = new_piece(1,0,2,2,true,true);
  game g_test = new_game(AR_WIDTH, AR_HEIGHT, 1, p_test);
  result2 = game_over_ar((cgame)g_test);
  delete_game(g_test);
  printf("test_game_over_ar : %d \n", result1 && result2);
  return (result1 && result2);
}

bool test_play_move(){
  bool result = true;
  for (int i=0; i<NB_PIECES; i++){
    if (can_move_x(pieces[i])){
      if (!play_move(g, i, LEFT, 1)){ //si la piece ne peut pas bouger, c'est qu'il y'a une autre piece ou bord du jeu atteint ; sinon, il y a un bug
	for (int j=0; j<NB_PIECES; j++){
	  if(i!=j){
	    result = intersect(pieces[i], pieces[j]); //si il y a intersect, tout est ok
	  }
	}
      }
      if (!play_move(g, i, RIGHT, 1)){
	for (int j=0; j<NB_PIECES; j++){
	  if(i!=j){
	    result = intersect(pieces[i], pieces[j]); //si il y a intersect, tout est ok
	  }
	}
      }
      result = !play_move(g, i, UP, 1);
      result = !play_move(g, i, DOWN, 1);
    }
    else{
      if (!play_move(g, i, UP, 1)){
	for (int j=0; j<NB_PIECES; j++){
	  if(i!=j){
	    result = intersect(pieces[i], pieces[j]);
	  }
	}
      }
      if (!play_move(g, i, DOWN, 1)){
	for (int j=0; j<NB_PIECES; j++){
	  if (i!=j){
	    result = intersect(pieces[i], pieces[j]);
	  }
	}
      }
      result = !play_move(g, i, LEFT, 1);
      result = !play_move(g, i, RIGHT, 1);
    }
  }
  printf("test_play_move : %d\n", result);
  return result;
}

bool test_game_square_piece(){ //ok c'est du code tout moche pas beau, mais la flemme de me rappeler du switch
  bool result = true;
  if (game_square_piece(g, 3, 3) != 0){
    printf("Echec test sur la piece 0");
    result = false;
  }
  if (game_square_piece(g, 3, 0) != 1){
    printf("Echec test sur la piece 1");
    result = false;
  }
  if (game_square_piece(g, 4, 1) != 2){
    printf("Echec test sur la piece 2");
    result = false;
  }
  if (game_square_piece(g, 5, 3) != 3){
    printf("Echec test sur la piece 3");
    result = false;
  }
  if (game_square_piece(g, 4, 4) != -1){
    printf("Echec test sur emplacement vide");
    result = false;
  }
  printf("test_square_piece : %d \n", result);
  return result;
}

bool test_game_width(){
  bool result = true;
  if (game_width((cgame)g) != AR_WIDTH){
    result = false;
  }
  printf("test_game_width : %d \n", result);
  return result;
}

bool test_game_height(){
  bool result = true;
  if (game_height((cgame)g) != AR_HEIGHT){
    result = false;
  }
  printf("test_game_height : %d \n", result);
  return result;
}

int main(int argc, char * argv[]){
  bool result = true;
  set_up();
  result = result && test_new_game();
  result = result && test_game_width();
  result = result && test_game_height();
  result = result && test_nb_pieces();
  result = result && test_game_piece();
  result = result && test_game_nb_moves();
  result = result && test_game_square_piece();
  result = result && test_copy_game();
  result = result && test_game_over_ar();
  result = result && test_play_move();
  delete_game(g);
  if (result){
    printf("Tests OK \n");
    return EXIT_SUCCESS;
  }
  else{
    printf("Tests failed \n");
    return EXIT_FAILURE;
  }
}
