#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include "piece.h"

#define NB_PIECES 4

/**
 * @brief test if value is equal to expected; if not, displays an error message containing msg to standard error output 
 * @return true if expected==value
 */
bool test_equality_int(int expected, int value, char * msg) {
  if (expected != value)
    fprintf(stderr,"ERR: value expected %d ; value computed %d. %s\n", expected, value, msg);
  return expected == value;
}

/**
 * @brief test if value is equal to expected; if not, displays an error message containing msg to standard error output 
 */
bool test_equality_bool(bool expected, bool value, char * msg) {
  if (expected != value)
    fprintf(stderr,"ERR: value expected %d ; value computed %d. %s\n", expected, value, msg);
  return expected == value;
}

piece pieces[NB_PIECES];
/* configue de test
111003..
...003..
..2223..
..2223..
.....3..
.....3..
 */

void set_up(){
  pieces[0] = new_piece(4, 4, 2, 2, false, false);
  pieces[1] = new_piece(0, 5, 3, 1, false, true);
  pieces[2] = new_piece(2, 2, 3, 2, true, true);
  pieces[3] = new_piece(6, 0, 1, 6, true, false);
}

void tear_down() {
  for (int i = 0 ; i < NB_PIECES; i++)
    delete_piece(pieces[i]);
}

bool test_new_piece() {
  bool result = true;
  for (int x = 0; x < 4; x++){
    for (int y = 0; y < 4; y++){
      for (int width = 1; width < 4; width++){
	for (int height = 1; height < 4; height++){
	  for (int move_x = 0; move_x <= 1; move_x++){
	    for (int move_y = 0; move_y <= 1; move_y++){
	      piece p = new_piece(x, y, width, height, (bool)move_x, (bool)move_y);
	      result = result && test_equality_int(x, get_x(p), "get_x");
	      result = result && test_equality_int(y, get_y(p),"get_y");
	      result = result && test_equality_int(width, get_width(p),"get_width");
	      result = result && test_equality_int(height, get_height(p),"get_height");
	      result = result && test_equality_bool((bool)move_x, can_move_x(p), "can_move_x");
	      result = result && test_equality_bool((bool)move_y, can_move_y(p), "can_move_y");
	      delete_piece(p);
	    }
	  }
	}
      }
    }
  }
  return result;
}

bool test_intersect() {
  bool result = true;
  set_up();
  for (int i=0; i < NB_PIECES; i++)
    for (int j =0; j<NB_PIECES; j++) {
      result = result && test_equality_bool(i==j, intersect(pieces[i], pieces[j]),"intersect");
    }

  piece piece1 = new_piece(3, 3, 2, 1, true, true);
  piece piece2 = new_piece(0, 1, 3, 2, false, false);
  result = result && test_equality_bool(true, intersect(pieces[2], piece1),"intersect piece1");
  result = result && test_equality_bool(true, intersect(piece2, pieces[2]),"intersect piece2");
  delete_piece(piece1);
  delete_piece(piece2);
  tear_down();
  return result;
}

bool test_move() {
  bool result = true;
  piece p = new_piece(0, 0, 1, 1, true, true);
  set_up();
  for (int dist = 1; dist < NB_PIECES; dist++)
    for (int i=0; i < NB_PIECES; i++) {
      copy_piece(pieces[i],p);
      move_piece(p, LEFT, dist);
      if (can_move_x(pieces[i]))
        result = result && test_equality_int(get_x(pieces[i])-dist,get_x(p),"move LEFT");
      else
        result = result && test_equality_int(get_x(pieces[i]),get_x(p),"move LEFT");
      copy_piece(pieces[i],p);
      move_piece(p, RIGHT, dist);
      if (can_move_x(pieces[i]))
        result = result && test_equality_int(get_x(pieces[i])+dist,get_x(p),"move RIGHT");
      else
        result = result && test_equality_int(get_x(pieces[i]),get_x(p),"move RIGHT");
      copy_piece(pieces[i],p);
      move_piece(p, UP, dist);
      if (can_move_y(pieces[i]))
        result = result && test_equality_int(get_y(pieces[i])+dist,get_y(p),"move UP");
      else
        result = result && test_equality_int(get_y(pieces[i]),get_y(p),"move UP");
      copy_piece(pieces[i],p);
      move_piece(p, DOWN, dist);
      if (can_move_y(pieces[i]))
        result = result && test_equality_int(get_y(pieces[i])-dist,get_y(p),"move DOWN");
      else
        result = result && test_equality_int(get_y(pieces[i]),get_y(p),"move DOWN");


    }
  tear_down();
  delete_piece(p);
  return result;
  return false;
}

bool test_copy() {
  piece p = new_piece(0, 0, 1, 1, true, true);
  bool result = true;
  set_up();
  for (int i = 0 ; i < NB_PIECES; i++) {
    copy_piece(pieces[i],p);
    result = result && test_equality_int(get_height(pieces[i]), get_height(p), "copy get_height");
    result = result && test_equality_int(get_width(pieces[i]), get_width(p), "copy get_width");
    result = result && test_equality_int(get_x(pieces[i]), get_x(p), "copy get_x");
    result = result && test_equality_int(get_y(pieces[i]), get_y(p), "copy get_y");
    result = result && test_equality_bool(can_move_x(pieces[i]), can_move_x(p), "copy can_move_x");
    result = result && test_equality_bool(can_move_y(pieces[i]), can_move_y(p), "copy can_move_y");
  }
  tear_down();
  delete_piece(p);
  return result;
}

int main (int argc, char *argv[])
{
  bool result= true;

  result = result && test_equality_bool(true, test_new_piece(), "new_piece");
  result = result && test_equality_bool(true, test_intersect(), "intersect");
  result = result && test_equality_bool(true, test_move(), "move");
  result = result && test_equality_bool(true, test_copy(), "copy");

  if (result){
    printf("réussis\n");
    return EXIT_SUCCESS;
  }else
    return EXIT_FAILURE;
}
